@@ -500,11 +500,10 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 	case 201:
 	case 301:
 	case 302:
+	case 303:
 		break;
 
 	case 206: /* write_queue is already prepared */
-		con->file_finished = 1;
-
 		break;
 	case 205: /* class: header only */
 	case 304:
@@ -970,7 +969,7 @@ int connection_handle_read_state(server *srv, connection *con)  {
 								}
 							} else {
 								/* a splited \r \n */
-								return -1;
+								break;
 							}
 						}
 					}
